@@ -309,6 +309,7 @@ struct drumkv1_aux
 struct drumkv1_gen
 {
 	float *sample, sample0;
+	float *reverse;
 	float *group;
 	float *coarse;
 	float *fine;
@@ -424,7 +425,7 @@ struct drumkv1_del
 	float *wet;
 	float *delay;
 	float *feedb;
-	float *bpm;
+	float *bpm, *bpm0;
 	float *bpmsync, bpmsync0;
 	float *bpmhost;
 };
@@ -781,6 +782,9 @@ public:
 
 	drumkv1_sample *sample() const;
 
+	void setReverse(bool bReverse);
+	bool isReverse() const;
+
 	void setParamPort(drumkv1::ParamIndex index, float *pfParam = 0);
 	float *paramPort(drumkv1::ParamIndex index);
 
@@ -892,6 +896,7 @@ drumkv1_impl::drumkv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 
 	// no delay sync yet
 	m_del.bpmsync0 = 0.0f;
+	m_del.bpm0 = 0;
 
 	// number of channels
 	setChannels(iChannels);
@@ -1122,6 +1127,18 @@ drumkv1_sample *drumkv1_impl::sample (void) const
 }
 
 
+void drumkv1_impl::setReverse ( bool bReverse )
+{
+	if (m_elem) m_elem->element.setReverse(bReverse);
+}
+
+
+bool drumkv1_impl::isReverse (void) const
+{
+	return (m_elem ? m_elem->element.isReverse() : false);
+}
+
+
 void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 {
 	static float s_fDummy = 0.0f;
@@ -1455,10 +1472,14 @@ void drumkv1_impl::resetParamValues ( bool bSwap )
 
 void drumkv1_impl::reset (void)
 {
-//--legacy support < 0.3.0.4 -- begin
+#if 1//--legacy support < 0.3.0.4
 	if (*m_del.bpm < 3.6f)
 		*m_del.bpm *= 100.0f;
-//--legacy support < 0.3.0.4 -- end.
+#endif
+
+	// make sure dangling states aren't...
+	m_del.bpmsync0 = 0.0f;
+	m_del.bpm0 = m_del.bpm;
 
 	// reset all elements
 	drumkv1_elem *elem = m_elem_list.next();
@@ -1518,6 +1539,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			elem->gen1.envtime0  = *elem->gen1.envtime;
 			elem->updateEnvTimes(m_iSampleRate);
 		}
+		elem->gen1_sample.reverse_test(*elem->gen1.reverse > 0.5f);
 		elem->lfo1_wave.reset_test(
 			drumkv1_wave::Shape(*elem->lfo1.shape), *elem->lfo1.width);
 		elem = elem->next();
@@ -1662,11 +1684,8 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	// delay sync toggle
 	if (int(*m_del.bpmsync) != int(m_del.bpmsync0)) {
-		float *del_bpm = m_del.bpm; 
-		float *del_bpmhost = m_del.bpmhost;
 		m_del.bpmsync0 = *m_del.bpmsync;
-		m_del.bpmhost = del_bpm;
-		m_del.bpm = del_bpmhost;
+		m_del.bpm0 = (m_del.bpmsync0 > 0.0f ? m_del.bpmhost : m_del.bpm);
 	}
 
 	// chorus
@@ -1686,7 +1705,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			*m_pha.rate, *m_pha.feedb, *m_pha.depth, *m_pha.daft * float(k));
 		// delay
 		m_delay[k].process(in, nframes, *m_del.wet,
-			*m_del.delay, *m_del.feedb, *m_del.bpm);
+			*m_del.delay, *m_del.feedb, *m_del.bpm0);
 	}
 
 	// reverb
@@ -1811,6 +1830,17 @@ drumkv1_sample *drumkv1::sample (void) const
 }
 
 
+void drumkv1::setReverse ( bool bReverse )
+{
+	m_pImpl->setReverse(bReverse);
+}
+
+bool drumkv1::isReverse (void) const
+{
+	return m_pImpl->isReverse();
+}
+
+
 void drumkv1::setParamPort ( ParamIndex index, float *pfParam )
 {
 	m_pImpl->setParamPort(index, pfParam);
@@ -1904,6 +1934,18 @@ drumkv1_sample *drumkv1_element::sample (void) const
 }
 
 
+void drumkv1_element::setReverse ( bool bReverse )
+{
+	if (m_pElem) m_pElem->gen1_sample.setReverse(bReverse);
+}
+
+
+bool drumkv1_element::isReverse (void) const
+{
+	return (m_pElem ? m_pElem->gen1_sample.isReverse() : false);
+}
+
+
 void drumkv1_element::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 {
 	if (m_pElem == 0)
@@ -1911,6 +1953,7 @@ void drumkv1_element::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 
 	switch (index) {
 //	case drumkv1::GEN1_SAMPLE:   m_pElem->gen1.sample      = pfParam; break;
+	case drumkv1::GEN1_REVERSE:  m_pElem->gen1.reverse     = pfParam; break;
 	case drumkv1::GEN1_GROUP:    m_pElem->gen1.group       = pfParam; break;
 	case drumkv1::GEN1_COARSE:   m_pElem->gen1.coarse      = pfParam; break;
 	case drumkv1::GEN1_FINE:     m_pElem->gen1.fine        = pfParam; break;
@@ -1959,6 +2002,7 @@ float *drumkv1_element::paramPort ( drumkv1::ParamIndex index )
 
 	switch (index) {
 //	case drumkv1::GEN1_SAMPLE:   pfParam = m_pElem->gen1.sample;     break;
+	case drumkv1::GEN1_REVERSE:  pfParam = m_pElem->gen1.reverse;    break;
 	case drumkv1::GEN1_GROUP:    pfParam = m_pElem->gen1.group;      break;
 	case drumkv1::GEN1_COARSE:   pfParam = m_pElem->gen1.coarse;     break;
 	case drumkv1::GEN1_FINE:     pfParam = m_pElem->gen1.fine;       break;
