@@ -19,7 +19,10 @@
 
 *****************************************************************************/
 
+#include "drumkv1_config.h"
+
 #include "drumkv1_jack.h"
+#include "drumkv1_param.h"
 
 #include <jack/midiport.h>
 
@@ -28,6 +31,9 @@
 
 #include <math.h>
 
+#include <QCoreApplication>
+#include <QDir>
+
 
 #ifdef CONFIG_ALSA_MIDI
 
@@ -105,6 +111,27 @@ int drumkv1_jack_process ( jack_nframes_t nframes, void *arg )
 }
 
 
+#ifdef CONFIG_JACK_SESSION
+
+#include <jack/session.h>
+
+#include <QDir>
+
+//----------------------------------------------------------------------
+// drumkv1_jack_session_event -- JACK session event callabck
+//
+
+static void drumkv1_jack_session_event (
+	jack_session_event_t *pSessionEvent, void *pvArg )
+{
+	drumkv1_jack *pDrumk = static_cast<drumkv1_jack *> (pvArg);
+
+	if (pDrumk)
+		pDrumk->sessionEvent(pSessionEvent);
+}
+
+#endif	// CONFIG_JACK_SESSION
+
 
 //-------------------------------------------------------------------------
 // drumkv1_jack - impl.
@@ -137,8 +164,8 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 
 	m_bpm = 0.0f;
 
-//	open(DRUMKV1_TITLE);
-//	activate();
+	open(DRUMKV1_TITLE);
+	activate();
 }
 
 
@@ -169,16 +196,16 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 			::jack_port_get_buffer(m_audio_outs[k], nframes));
 	}
 
-	const float *bpm_sync = drumkv1::paramPort(drumkv1::DEL1_BPMSYNC);
-	if (bpm_sync && *bpm_sync > 0.0f) {
-		float *bpm_port = drumkv1::paramPort(drumkv1::DEL1_BPM);
-		if (bpm_port) {
+	const float *bpmsync = paramPort(drumkv1::DEL1_BPMSYNC);
+	if (bpmsync && *bpmsync > 0.0f) {
+		float *bpmhost = paramPort(drumkv1::DEL1_BPMHOST);
+		if (bpmhost) {
 			jack_position_t pos;
 			jack_transport_query(m_client, &pos);
 			if (pos.valid & JackPositionBBT) {
 				const float bpm = float(pos.beats_per_minute);
-				if (::fabs(*bpm_port - bpm) > 0.01f)
-					*bpm_port = bpm;
+				if (::fabs(*bpmhost - bpm) > 0.01f)
+					*bpmhost = bpm;
 			}
 		}
 	}
@@ -245,8 +272,11 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 void drumkv1_jack::open ( const char *client_id )
 {
 	// init param ports
-	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
-		drumkv1::setParamPort(drumkv1::ParamIndex(i), &m_params[i]);
+	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
+		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		m_params[i] = drumkv1_param::paramDefaultValue(index);
+		drumkv1::setParamPort(index, &m_params[i]);
+	}
 
 	// open client
 	m_client = ::jack_client_open(client_id, JackNullOption, NULL);
@@ -268,11 +298,11 @@ void drumkv1_jack::open ( const char *client_id )
 
 	char port_name[32];
 	for (uint16_t k = 0; k < nchannels; ++k) {
-		::snprintf(port_name, sizeof(port_name), "in_%d", k);
+		::snprintf(port_name, sizeof(port_name), "in_%d", k + 1);
 		m_audio_ins[k] = ::jack_port_register(m_client,
 			port_name, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
 		m_ins[k] = NULL;
-		::snprintf(port_name, sizeof(port_name), "out_%d", k);
+		::snprintf(port_name, sizeof(port_name), "out_%d", k + 1);
 		m_audio_outs[k] = ::jack_port_register(m_client,
 			port_name, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 		m_outs[k] = NULL;
@@ -308,6 +338,14 @@ void drumkv1_jack::open ( const char *client_id )
 	// set process callbacks...
 	::jack_set_process_callback(m_client,
 		drumkv1_jack_process, this);
+
+#ifdef CONFIG_JACK_SESSION
+	// JACK session event callback...
+	if (::jack_set_session_callback) {
+		::jack_set_session_callback(m_client,
+			drumkv1_jack_session_event, this);
+	}
+#endif
 }
 
 
@@ -509,4 +547,323 @@ void drumkv1_jack::alsa_capture ( snd_seq_event_t *ev )
 #endif	// CONFIG_ALSA_MIDI
 
 
+#ifdef CONFIG_JACK_SESSION
+
+// JACK session event handler.
+void drumkv1_jack::sessionEvent ( void *pvSessionArg )
+{
+	jack_session_event_t *pJackSessionEvent
+		= (jack_session_event_t *) pvSessionArg;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1_jack::sessionEvent()"
+		" type=%d client_uuid=\"%s\" session_dir=\"%s\"",
+		int(pJackSessionEvent->type),
+		pJackSessionEvent->client_uuid,
+		pJackSessionEvent->session_dir);
+#endif
+
+	const bool bQuit = (pJackSessionEvent->type == JackSessionSaveAndQuit);
+
+	const QString sSessionDir
+		= QString::fromUtf8(pJackSessionEvent->session_dir);
+	const QString sSessionName
+		= QFileInfo(QFileInfo(sSessionDir).canonicalPath()).completeBaseName();
+	const QString sSessionFile = sSessionName + '.' + DRUMKV1_TITLE;
+
+	QStringList args;
+	args << QCoreApplication::applicationFilePath();
+	args << QString("\"${SESSION_DIR}%1\"").arg(sSessionFile);
+
+	drumkv1_param::savePreset(this,
+		QFileInfo(sSessionDir, sSessionFile).absoluteFilePath());
+
+	const QByteArray aCmdLine = args.join(" ").toUtf8();
+	pJackSessionEvent->command_line = ::strdup(aCmdLine.constData());
+
+	jack_session_reply(m_client, pJackSessionEvent);
+	jack_session_event_free(pJackSessionEvent);
+
+	if (bQuit)
+		QCoreApplication::quit();
+}
+
+#endif	// CONFIG_JACK_SESSION
+
+
+//-------------------------------------------------------------------------
+// drumkv1_application -- Singleton application instance.
+//
+
+#include "drumkv1widget_jack.h"
+
+#include <QApplication>
+#include <QTextStream>
+
+#ifdef CONFIG_NSM
+#include "drumkv1_nsm.h"
+#endif
+
+
+// Constructor.
+drumkv1_application::drumkv1_application ( int& argc, char **argv )
+	: QObject(NULL), m_pApp(NULL), m_bGui(true),
+		m_pDrumk(NULL), m_pWidget(NULL)
+	  #ifdef CONFIG_NSM
+		, m_pNsmClient(NULL)
+	  #endif
+{
+#ifdef Q_WS_X11
+	m_bGui = (::getenv("DISPLAY") != 0);
+#endif
+	for (int i = 1; i < argc; ++i) {
+		const QString& sArg = QString::fromLocal8Bit(argv[i]);
+		if (sArg[0] != '-')
+			m_presets.append(sArg);
+		else
+		if (sArg == "-g" || sArg == "--no-gui")
+			m_bGui = false;
+	}
+
+	if (m_bGui)
+		m_pApp = new QApplication(argc, argv);
+	else
+		m_pApp = new QCoreApplication(argc, argv);
+}
+
+
+// Destructor.
+drumkv1_application::~drumkv1_application (void)
+{
+#ifdef CONFIG_NSM
+	if (m_pNsmClient) delete m_pNsmClient;
+#endif
+	if (m_pWidget) delete m_pWidget;
+	if (m_pDrumk) delete m_pDrumk;
+	if (m_pApp) delete m_pApp;
+}
+
+
+// Argument parser method.
+bool drumkv1_application::parse_args (void)
+{
+	QTextStream out(stderr);
+
+	const QStringList& args = m_pApp->arguments();
+	QStringListIterator iter(args);
+	while (iter.hasNext()) {
+		const QString& sArg = iter.next();
+		if (sArg == "-h" || sArg == "--help") {
+			out << QObject::tr(
+				"Usage: %1 [options] [preset-file]\n\n"
+				DRUMKV1_TITLE " - " DRUMKV1_SUBTITLE "\n\n"
+				"Options:\n\n"
+				"  -g, --no-gui\n\tDisable the graphical user interface (GUI)\n\n"
+				"  -h, --help\n\tShow help about command line options\n\n"
+				"  -v, --version\n\tShow version information\n\n")
+				.arg(args.at(0));
+			return false;
+		}
+		else
+		if (sArg == "-v" || sArg == "-V" || sArg == "--version") {
+			out << QObject::tr("Qt: %1\n").arg(qVersion());
+			out << QObject::tr(DRUMKV1_TITLE ": %1\n").arg(DRUMKV1_VERSION);
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
+// Startup methods.
+bool drumkv1_application::setup (void)
+{
+	if (m_pApp == NULL)
+		return false;
+
+	if (!parse_args()) {
+		m_pApp->quit();
+		return false;
+	}
+
+	m_pDrumk = new drumkv1_jack();
+
+	if (m_bGui) {
+		m_pWidget = new drumkv1widget_jack(m_pDrumk);
+		if (m_presets.isEmpty())
+			m_pWidget->initPreset();
+		else
+			m_pWidget->loadPreset(m_presets.first());
+		m_pWidget->show();
+	}
+	else
+	if (!m_presets.isEmpty())
+		drumkv1_param::loadPreset(m_pDrumk, m_presets.first());
+
+#ifdef CONFIG_NSM
+	// Check whether to participate into a NSM session...
+	const QString& nsm_url
+		= QString::fromLatin1(::getenv("NSM_URL"));
+	if (!nsm_url.isEmpty()) {
+		m_pNsmClient = new drumkv1_nsm(nsm_url);
+		QObject::connect(m_pNsmClient,
+			SIGNAL(open()),
+			SLOT(openSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(save()),
+			SLOT(saveSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(show()),
+			SLOT(showSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(hide()),
+			SLOT(hideSession()));
+		QString caps(":switch:dirty:");
+		if (m_bGui)
+			caps += "optional-gui:";
+		m_pNsmClient->announce(DRUMKV1_TITLE, caps.toLatin1().constData());
+		if (m_pWidget)
+			m_pWidget->setNsmClient(m_pNsmClient);
+	}
+#endif	// CONFIG_NSM
+
+	return true;
+}
+
+
+// Facade method.
+int drumkv1_application::exec (void)
+{
+	return (setup() ? m_pApp->exec() : 1);
+}
+
+
+#ifdef CONFIG_NSM
+
+void drumkv1_application::openSession (void)
+{
+	if (m_pDrumk == NULL)
+		return;
+
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1_jack::openSession()");
+#endif
+
+	m_pDrumk->deactivate();
+	m_pDrumk->close();
+
+	const QString& client_id = m_pNsmClient->client_id();
+	const QString& path_name = m_pNsmClient->path_name();
+	const QString& display_name = m_pNsmClient->display_name();
+
+	m_pDrumk->open(client_id.toUtf8().constData());
+	m_pDrumk->activate();
+
+	const QDir dir(path_name);
+	if (!dir.exists())
+		dir.mkpath(path_name);
+
+	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
+	if (fi.exists()) {
+		const QString& sFilename = fi.absoluteFilePath();
+		if (m_pWidget) {
+			m_pWidget->loadPreset(sFilename);
+		} else {
+			drumkv1_param::loadPreset(m_pDrumk, sFilename);
+		}
+	}
+
+	m_pNsmClient->open_reply();
+	m_pNsmClient->dirty(false);
+
+	if (m_pWidget)
+		m_pNsmClient->visible(m_pWidget->isVisible());
+}
+
+void drumkv1_application::saveSession (void)
+{
+	if (m_pDrumk == NULL)
+		return;
+
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1_jack::saveSession()");
+#endif
+
+	const QString& path_name = m_pNsmClient->path_name();
+	const QString& display_name = m_pNsmClient->display_name();
+//	const QString& client_id = m_pNsmClient->client_id();
+	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
+
+	drumkv1_param::savePreset(m_pDrumk, fi.absoluteFilePath());
+
+	m_pNsmClient->save_reply();
+	m_pNsmClient->dirty(false);
+}
+
+
+void drumkv1_application::showSession (void)
+{
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1_jack::showSession()");
+#endif
+
+	if (m_pWidget) {
+		m_pWidget->show();
+		m_pWidget->raise();
+		m_pWidget->activateWindow();
+	}
+}
+
+void drumkv1_application::hideSession (void)
+{
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1_jack::hideSession()");
+#endif
+
+	if (m_pWidget)
+		m_pWidget->hide();
+}
+
+
+#endif	// CONFIG_NSM
+
+
+//-------------------------------------------------------------------------
+// main
+
+int main ( int argc, char *argv[] )
+{
+	Q_INIT_RESOURCE(drumkv1);
+
+	drumkv1_application app(argc, argv);
+
+	return app.exec();
+}
+
+
 // end of drumkv1_jack.cpp
