@@ -1,7 +1,7 @@
 // drumkv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,19 +22,17 @@
 #include "drumkv1_lv2.h"
 #include "drumkv1_config.h"
 
+#include "drumkv1_sched.h"
 #include "drumkv1_param.h"
 
-#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
+#include "lv2/lv2plug.in/ns/ext/time/time.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
 
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
 
 #include <stdlib.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/socket.h>
+#include <math.h>
 
 #include <QDomDocument>
 
@@ -103,14 +101,23 @@ drumkv1_lv2::drumkv1_lv2 (
 	: drumkv1(2, uint32_t(sample_rate))
 {
 	m_urid_map = NULL;
-	m_midi_event_type = 0;
 	m_atom_sequence = NULL;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		if (::strcmp(host_features[i]->URI, LV2_URID_MAP_URI) == 0) {
 			m_urid_map = (LV2_URID_Map *) host_features[i]->data;
 			if (m_urid_map) {
-				m_midi_event_type = m_urid_map->map(
+				m_urids.atom_Blank = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Blank);
+				m_urids.atom_Object = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Object);
+				m_urids.atom_Float = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Float);
+				m_urids.time_Position = m_urid_map->map(
+					m_urid_map->handle, LV2_TIME__Position);
+				m_urids.time_beatsPerMinute = m_urid_map->map(
+					m_urid_map->handle, LV2_TIME__beatsPerMinute);
+				m_urids.midi_MidiEvent = m_urid_map->map(
 					m_urid_map->handle, LV2_MIDI__MidiEvent);
 				break;
 			}
@@ -122,9 +129,6 @@ drumkv1_lv2::drumkv1_lv2 (
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
 		m_ins[k] = m_outs[k] = NULL;
-
-	::socketpair(AF_UNIX, SOCK_STREAM, 0, m_update_fds);
-	m_update_count = 0;
 }
 
 
@@ -173,9 +177,11 @@ void drumkv1_lv2::run ( uint32_t nframes )
 
 	if (m_atom_sequence) {
 		LV2_ATOM_SEQUENCE_FOREACH(m_atom_sequence, event) {
-			if (event && event->body.type == m_midi_event_type) {
+			if (event == NULL)
+				continue;
+			if (event->body.type == m_urids.midi_MidiEvent) {
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
-				uint32_t nread = event->time.frames - ndelta;
+				const uint32_t nread = event->time.frames - ndelta;
 				if (nread > 0) {
 					process(ins, outs, nread);
 					for (uint16_t k = 0; k < nchannels; ++k) {
@@ -186,6 +192,28 @@ void drumkv1_lv2::run ( uint32_t nframes )
 				ndelta = event->time.frames;
 				process_midi(data, event->body.size);
 			}
+			else
+			if (event->body.type == m_urids.atom_Blank ||
+				event->body.type == m_urids.atom_Object) {
+				const LV2_Atom_Object *object
+					= (LV2_Atom_Object *) &event->body;
+				if (object->body.otype == m_urids.time_Position) {
+					LV2_Atom *atom = NULL;
+					lv2_atom_object_get(object,
+						m_urids.time_beatsPerMinute, &atom, NULL);
+					if (atom && atom->type == m_urids.atom_Float) {
+						const float *bpmsync = paramPort(drumkv1::DEL1_BPMSYNC);
+						if (bpmsync && *bpmsync > 0.0f) {
+							float *bpmhost = paramPort(drumkv1::DEL1_BPMHOST);
+							if (bpmhost) {
+								const float bpm	= ((LV2_Atom_Float *) atom)->body;
+								if (::fabs(*bpmhost - bpm) > 0.01f)
+									*bpmhost = bpm;
+							}
+						}
+					}
+				}
+			}
 		}
 	//	m_atom_sequence = NULL;
 	}
@@ -212,32 +240,6 @@ uint32_t drumkv1_lv2::urid_map ( const char *uri ) const
 }
 
 
-int drumkv1_lv2::update_fds ( int mode ) const
-{
-	return m_update_fds[mode];
-}
-
-
-void drumkv1_lv2::update_notify (void)
-{
-	if (m_update_count < 1) {
-		char c = 1;
-		if (::write(m_update_fds[0], &c, sizeof(c)) > 0)
-			++m_update_count;
-	}
-}
-
-
-void drumkv1_lv2::update_reset (void)
-{
-	if (m_update_count > 0) {
-		char c;
-		if (::read(m_update_fds[1], &c, sizeof(c)) > 0)
-			m_update_count = 0;
-	}
-}
-
-
 //-------------------------------------------------------------------------
 // drumkv1_lv2 - LV2 State interface.
 //
@@ -322,6 +324,8 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 			drumkv1_param::loadElements(pPlugin, eElements, mapPath);
 	}
 
+	drumkv1_sched::sync_notify();
+
 	return LV2_STATE_SUCCESS;
 }
 
