@@ -1,7 +1,7 @@
 // drumkv1_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -63,24 +63,25 @@ public:
 
 	uint32_t urid_map(const char *uri) const;
 
-	int update_fds(int mode) const;
-
-	void update_notify();
-	void update_reset();
-
 private:
 
 	LV2_URID_Map *m_urid_map;
 
-	uint32_t m_midi_event_type;
+	struct lv2_urids
+	{
+		LV2_URID atom_Blank;
+		LV2_URID atom_Object;
+		LV2_URID atom_Float;
+		LV2_URID time_Position;
+		LV2_URID time_beatsPerMinute;
+		LV2_URID midi_MidiEvent;
+
+	} m_urids;
 
 	LV2_Atom_Sequence *m_atom_sequence;
 
 	float **m_ins;
 	float **m_outs;
-
-	int m_update_fds[2];
-	int m_update_count;
 };
 
 
