@@ -25,7 +25,6 @@
 #include "drumkv1.h"
 
 #include <QString>
-#include <QHash>
 
 // forward decl.
 class QDomElement;
@@ -43,10 +42,8 @@ namespace drumkv1_param
 	{
 	public:
 
-		virtual QString absolutePath(const QString& sAbstractPath) const
-			{ return sAbstractPath; }
-		virtual QString abstractPath(const QString& sAbsolutePath) const
-			{ return sAbsolutePath; }
+		virtual QString absolutePath(const QString& sAbstractPath) const;
+		virtual QString abstractPath(const QString& sAbsolutePath) const;
 	};
 
 	// Element serialization methods.
@@ -57,6 +54,12 @@ namespace drumkv1_param
 		QDomDocument& doc, QDomElement& eElements,
 		const map_path& mapPath = map_path());
 
+	// Preset serialization methods.
+	void loadPreset(drumkv1 *pDrumk,
+		const QString& sFilename);
+	void savePreset(drumkv1 *pDrumk,
+		const QString& sFilename);
+
 	// Default parameter name/value helpers.
 	const char *paramName(drumkv1::ParamIndex index);
 	float paramDefaultValue(drumkv1::ParamIndex index);
