@@ -27,7 +27,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <sndfile.h>
+// forward decls.
+class drumkv1_reverse_sched;
 
 
 //-------------------------------------------------------------------------
@@ -39,14 +40,10 @@ class drumkv1_sample
 public:
 
 	// ctor.
-	drumkv1_sample(float srate = 44100.0f)
-		: m_srate(srate), m_filename(0), m_nchannels(0),
-			m_rate0(0.0f), m_freq0(1.0f), m_ratio(0.0f),
-			m_nframes(0), m_pframes(0) {}
+	drumkv1_sample(float srate = 44100.0f);
 
 	// dtor.
-	~drumkv1_sample()
-		{ close(); }
+	~drumkv1_sample();
 
 	// nominal sample-rate.
 	void setSampleRate(float srate)
@@ -54,75 +51,33 @@ public:
 	float sampleRate() const
 		{ return m_srate; }
 
+	// reverse mode.
+	void setReverse (bool reverse)
+	{
+		if (( m_reverse && !reverse) ||
+			(!m_reverse &&  reverse)) {
+			m_reverse = reverse;
+			reverse_sample();
+		}
+	}
+
+	bool isReverse() const
+		{ return m_reverse; }
+
+	// schedule sample reverse.
+	void reverse_test(bool reverse)
+	{
+		if (( m_reverse && !reverse) ||
+			(!m_reverse &&  reverse)) {
+			reverse_sched(reverse);
+		}
+	}
+
+	void reverse_sched(bool reverse);
+
 	// init.
-	bool open(const char *filename, float freq0 = 1.0f)
-	{
-		if (!filename)
-			return false;
-
-		close();
-
-		m_filename = ::strdup(filename);
-
-		SF_INFO info;
-		::memset(&info, 0, sizeof(info));
-		
-		SNDFILE *file = ::sf_open(m_filename, SFM_READ, &info);
-		if (!file)
-			return false;
-
-		m_nchannels = info.channels;
-		m_rate0     = float(info.samplerate);
-		m_nframes   = info.frames;
-
-		const uint32_t nsize = m_nframes + 4;
-
-		m_pframes = new float * [m_nchannels];
-		for (uint16_t k = 0; k < m_nchannels; ++k) {
-			m_pframes[k] = new float [nsize];
-			::memset(m_pframes[k], 0, nsize * sizeof(float));
-		}
-
-		float *buffer = new float [m_nchannels * m_nframes];
-
-		const int nread = ::sf_readf_float(file, buffer, m_nframes);
-		if (nread > 0) {
-			const uint32_t n = uint32_t(nread);
-			uint32_t i = 0;
-			for (uint32_t j = 0; j < n; ++j) {
-				for (uint16_t k = 0; k < m_nchannels; ++k)
-					m_pframes[k][j] = buffer[i++];
-			}
-		}
-	
-		delete [] buffer;
-		::sf_close(file);
-
-		reset(freq0);
-
-		return true;
-	}
-
-	void close()
-	{
-		if (m_pframes) {
-			for (uint16_t k = 0; k < m_nchannels; ++k)
-				delete [] m_pframes[k];
-			delete [] m_pframes;
-			m_pframes = 0;
-		}
-
-		m_nframes   = 0;
-		m_ratio     = 0.0f;
-		m_freq0     = 1.0f;
-		m_rate0     = 0.0f;
-		m_nchannels = 0;
-
-		if (m_filename) {
-			::free(m_filename);
-			m_filename = 0;
-		}
-	}
+	bool open(const char *filename, float freq0 = 1.0f);
+	void close();
 
 	// accessors.
 	const char *filename() const
@@ -155,6 +110,11 @@ public:
 	bool isOver(uint32_t frame) const
 		{ return !m_pframes || (frame >= m_nframes); }
 
+protected:
+
+	// reverse sample buffer.
+	void reverse_sample();
+
 private:
 
 	float    m_srate;
@@ -165,6 +125,9 @@ private:
 	float    m_ratio;
 	uint32_t m_nframes;
 	float  **m_pframes;
+	bool     m_reverse;
+
+	drumkv1_reverse_sched *m_reverse_sched;
 };
 
 
