@@ -70,8 +70,10 @@ private:
 };
 
 
-static drumkv1_sched_thread *g_sched_thread   = NULL;
-static uint32_t              g_sched_refcount = 0;
+static drumkv1_sched_thread *g_sched_thread = NULL;
+static uint32_t g_sched_refcount = 0;
+
+static QList<drumkv1_sched_notifier *> g_sched_notifiers;
 
 
 //-------------------------------------------------------------------------
@@ -99,7 +101,7 @@ drumkv1_sched_thread::drumkv1_sched_thread ( uint32_t nsize ) : QThread()
 // dtor.
 drumkv1_sched_thread::~drumkv1_sched_thread (void)
 {
-	// fake sync and wait 
+	// fake sync and wait
 	if (m_running && isRunning()) do {
 		if (m_mutex.tryLock()) {
 			m_running = false;
@@ -162,7 +164,8 @@ void drumkv1_sched_thread::run (void)
 //
 
 // ctor.
-drumkv1_sched::drumkv1_sched (void) : m_sync_wait(false)
+drumkv1_sched::drumkv1_sched (void)
+	: m_sync_wait(false)
 {
 	if (++g_sched_refcount == 1 && g_sched_thread == NULL) {
 		g_sched_thread = new drumkv1_sched_thread();
@@ -174,9 +177,11 @@ drumkv1_sched::drumkv1_sched (void) : m_sync_wait(false)
 // dtor (virtual).
 drumkv1_sched::~drumkv1_sched (void)
 {
-	if (--g_sched_refcount == 0 && g_sched_thread) {
-		delete g_sched_thread;
-		g_sched_thread = NULL;
+	if (--g_sched_refcount == 0) {
+		if (g_sched_thread) {
+			delete g_sched_thread;
+			g_sched_thread = NULL;
+		}
 	}
 }
 
@@ -207,6 +212,42 @@ void drumkv1_sched::sync_process (void)
 	process();
 
 	m_sync_wait = false;
+
+	sync_notify();
+}
+
+
+// signal broadcast (static).
+void drumkv1_sched::sync_notify (void)
+{
+	QListIterator<drumkv1_sched_notifier *> iter(g_sched_notifiers);
+	while (iter.hasNext())
+		iter.next()->sync_notify();
+}
+
+
+//-------------------------------------------------------------------------
+// drumkv1_sched_notifier - worker/schedule proxy decl.
+//
+
+// ctor.
+drumkv1_sched_notifier::drumkv1_sched_notifier ( QObject *parent )
+	: QObject(parent)
+{
+	g_sched_notifiers.append(this);
+}
+
+
+// dtor.
+drumkv1_sched_notifier::~drumkv1_sched_notifier (void)
+{
+	g_sched_notifiers.removeAll(this);
+}
+
+
+void drumkv1_sched_notifier::sync_notify (void)
+{
+	emit notify();
 }
 
 
