@@ -23,6 +23,10 @@
 #define __drumkv1_sched_h
 
 
+// forward decls.
+class drumkv1_sched_notifier;
+
+
 //-------------------------------------------------------------------------
 // drumkv1_sched - worker/scheduled stuff (pure virtual).
 //
@@ -49,6 +53,9 @@ public:
 	// (pure) virtual processor.
 	virtual void process() = 0;
 
+	// signal broadcast (static).
+	static void sync_notify();
+
 private:
 
 	// instance variables.
@@ -56,6 +63,33 @@ private:
 };
 
 
+//-------------------------------------------------------------------------
+// drumkv1_sched_notifier - worker/schedule proxy decl.
+//
+
+#include <QObject>
+
+class drumkv1_sched_notifier : public QObject
+{
+	Q_OBJECT
+
+public:
+
+	// ctor.
+	drumkv1_sched_notifier(QObject *parent = NULL);
+
+	// dtor.
+	~drumkv1_sched_notifier();
+
+	// signal notifier.
+	void sync_notify();
+
+signals:
+
+	void notify();
+};
+
+
 #endif	// __drumkv1_sched_h
 
 // end of drumkv1_sched.h
