@@ -63,6 +63,9 @@ void drumkv1_wave::reset ( Shape shape, float width )
 	case Sine:
 		reset_sine();
 		break;
+	case Rand:
+		reset_rand();
+		break;
 	case Noise:
 		reset_noise();
 		// thru...
@@ -134,8 +137,8 @@ void drumkv1_wave::reset_sine (void)
 }
 
 
-// init noise table.
-void drumkv1_wave::reset_noise (void)
+// init random table.
+void drumkv1_wave::reset_rand (void)
 {
 	const float p0 = float(m_nsize);
 	const float w0 = p0 * m_width;
@@ -157,6 +160,23 @@ void drumkv1_wave::reset_noise (void)
 }
 
 
+// init noise table.
+void drumkv1_wave::reset_noise (void)
+{
+	const float p0 = float(m_nsize);
+	const float w0 = p0 * m_width;
+
+	m_srand = uint32_t(w0) ^ 0x9631; // magic!
+
+	for (uint32_t i = 0; i < m_nsize; ++i)
+		m_table[i] = pseudo_randf();
+
+//	reset_filter();
+//	reset_normalize();
+	reset_interp();
+}
+
+
 // post-processors
 void drumkv1_wave::reset_filter (void)
 {
