@@ -22,9 +22,10 @@
 #include "drumkv1widget.h"
 #include "drumkv1_param.h"
 
-#include <QDomDocument>
-#include <QTextStream>
-#include <QFileInfo>
+#include "drumkv1_sample.h"
+#include "drumkv1_sched.h"
+
+#include "drumkv1widget_config.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -61,6 +62,9 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.setupUi(this);
 
+	// Init sched notifier.
+	m_sched_notifier = new drumkv1_sched_notifier(this);
+
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
 		m_params_ab[i] = drumkv1_param::paramDefaultValue(drumkv1::ParamIndex(i));
@@ -84,6 +88,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	shapes << tr("Pulse");
 	shapes << tr("Saw");
 	shapes << tr("Sine");
+	shapes << tr("Rand");
 	shapes << tr("Noise");
 
 	m_ui.Lfo1ShapeKnob->insertItems(0, shapes);
@@ -124,6 +129,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	states << tr("Off");
 	states << tr("On");
 
+	m_ui.Gen1ReverseKnob->insertItems(0, states);
+
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
 
@@ -139,9 +146,16 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Gen1EnvTimeKnob->setSpecialValueText(sAuto);
 	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
+	// Wave integer widths.
+	m_ui.Lfo1WidthKnob->setDecimals(0);
+
 	// GEN group limits. [0=off, 1..128]
 	m_ui.Gen1GroupKnob->setSpecialValueText(sOff);
-	m_ui.Gen1GroupKnob->setMaximum(1.28f);
+	m_ui.Gen1GroupKnob->setScale(1.0f);
+	m_ui.Gen1GroupKnob->setMinimum(0.0f);
+	m_ui.Gen1GroupKnob->setMaximum(128.0f);
+	m_ui.Gen1GroupKnob->setDecimals(0);
+	m_ui.Gen1GroupKnob->setSingleStep(10.0f);
 
 	// GEN octave limits.
 	m_ui.Gen1CoarseKnob->setMinimum(-4.0f);
@@ -181,8 +195,10 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setScale(1.0f);
 	m_ui.Del1BpmKnob->setMinimum(3.6f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
+	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
 	// GEN1
+	setParamKnob(drumkv1::GEN1_REVERSE, m_ui.Gen1ReverseKnob);
 	setParamKnob(drumkv1::GEN1_GROUP,   m_ui.Gen1GroupKnob);
 	setParamKnob(drumkv1::GEN1_COARSE,  m_ui.Gen1CoarseKnob);
 	setParamKnob(drumkv1::GEN1_FINE,    m_ui.Gen1FineKnob);
@@ -446,7 +462,14 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
+	if (pConfig)
+		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
+
 	// Menu actions
+	QObject::connect(m_ui.helpUseNativeDialogsAction,
+		SIGNAL(triggered(bool)),
+		SLOT(helpUseNativeDialogs(bool)));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -454,6 +477,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
+	// Special sample update notifications (eg. reverse)
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify()),
+		SLOT(updateSampleNotify()));
+
 	// Epilog.
 	// QWidget::adjustSize();
 
@@ -463,6 +491,13 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 }
 
 
+// Destructor.
+drumkv1widget::~drumkv1widget (void)
+{
+	delete m_sched_notifier;
+}
+
+
 // Param kbob (widget) map accesors.
 void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_knob *pKnob )
 {
@@ -540,9 +575,21 @@ void drumkv1widget::paramChanged ( float fValue )
 // Update local tied widgets.
 void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 {
+	drumkv1 *pDrumk = instance();
+	if (pDrumk == NULL)
+		return;
+
 	++m_iUpdate;
 
 	switch (index) {
+#if 0//--updateSampleNotify();
+	case drumkv1::GEN1_REVERSE: {
+		const bool bReverse = bool(fValue > 0.0f);
+		pDrumk->setReverse(bReverse);
+		updateSample(pDrumk->sample());
+		break;
+	}
+#endif
 	case drumkv1::DEL1_BPMSYNC:
 		if (fValue > 0.0f)
 			m_ui.Del1BpmKnob->setValue(0.0f);
@@ -645,6 +692,28 @@ void drumkv1widget::resetSwapParams (void)
 }
 
 
+// Initialize param values.
+void drumkv1widget::updateParamValues ( uint32_t nparams )
+{
+	resetSwapParams();
+
+	drumkv1 *pDrumk = instance();
+
+	for (uint32_t i = 0; i < nparams; ++i) {
+		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		float fValue = drumkv1_param::paramDefaultValue(index);
+		const float *pfParamPort
+			= (pDrumk ? pDrumk->paramPort(index) : NULL);
+		if (pfParamPort)
+			fValue = *pfParamPort;
+		setParamValue(index, fValue, true);
+		updateParam(index, fValue);
+	//	updateParamEx(index, fValue);
+		m_params_ab[index] = fValue;
+	}
+}
+
+
 // Reset all param default values.
 void drumkv1widget::resetParamValues ( uint32_t nparams )
 {
@@ -718,18 +787,6 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 	qDebug("drumkv1widget::loadPreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	QFile file(sFilename);
-	if (!file.open(QIODevice::ReadOnly))
-		return;
-
-	static QHash<QString, drumkv1::ParamIndex> s_hash;
-	if (s_hash.isEmpty()) {
-		for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i) {
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			s_hash.insert(drumkv1_param::paramName(index), index);
-		}
-	}
-
 	clearElements();
 
 	clearSampleFile();
@@ -737,66 +794,21 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 	resetParamKnobs(drumkv1::NUM_PARAMS);
 	resetParamValues(drumkv1::NUM_PARAMS);
 
-	const QFileInfo fi(sFilename);
-	const QDir currentDir(QDir::current());
-	QDir::setCurrent(fi.absolutePath());
+	drumkv1 *pDrumk = instance();
+	if (pDrumk == NULL)
+		return;
 
-	QDomDocument doc(DRUMKV1_TITLE);
-	if (doc.setContent(&file)) {
-		QDomElement ePreset = doc.documentElement();
-		if (ePreset.tagName() == "preset"
-			&& ePreset.attribute("name") == fi.completeBaseName()) {
-			for (QDomNode nChild = ePreset.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "elements") {
-					drumkv1_param::loadElements(instance(), eChild);
-				}
-				else
-				if (eChild.tagName() == "params") {
-					for (QDomNode nParam = eChild.firstChild();
-							!nParam.isNull();
-								nParam = nParam.nextSibling()) {
-						QDomElement eParam = nParam.toElement();
-						if (eParam.isNull())
-							continue;
-						if (eParam.tagName() == "param") {
-							drumkv1::ParamIndex index = drumkv1::ParamIndex(
-								eParam.attribute("index").toULong());
-							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty()) {
-								if (!s_hash.contains(sName))
-									continue;
-								index = s_hash.value(sName);
-							}
-							float fValue = eParam.text().toFloat();
-						//--legacy support < 0.3.0.4 -- begin
-							if (index == drumkv1::DEL1_BPM && fValue < 3.6f)
-								fValue *= 100.0f;
-						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue, true);
-							updateParam(index, fValue);
-							m_params_ab[index] = fValue;
-						}
-					}
-				}
-			}
-		}
-	}
+	drumkv1_param::loadPreset(pDrumk, sFilename);
 
-	file.close();
+	updateParamValues(drumkv1::NUM_PARAMS);
+
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
-	const QString& sPreset = fi.completeBaseName();
 	m_ui.Preset->setPreset(sPreset);
-
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
 
-	QDir::setCurrent(currentDir.absolutePath());
-
 	refreshElements();
 	activateElement();
 }
@@ -807,44 +819,14 @@ void drumkv1widget::savePreset ( const QString& sFilename )
 #ifdef CONFIG_DEBUG
 	qDebug("drumkv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
-	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
 
-	const QFileInfo fi(sFilename);
-	const QDir currentDir(QDir::current());
-	QDir::setCurrent(fi.absolutePath());
+	drumkv1_param::savePreset(instance(), sFilename);
 
-	QDomDocument doc(DRUMKV1_TITLE);
-	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", sPreset);
-	ePreset.setAttribute("version", DRUMKV1_VERSION);
-
-	QDomElement eElements = doc.createElement("elements");
-	drumkv1_param::saveElements(instance(), doc, eElements);
-	ePreset.appendChild(eElements);
-
-	QDomElement eParams = doc.createElement("params");
-	for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i) {
-		QDomElement eParam = doc.createElement("param");
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", drumkv1_param::paramName(index));
-		eParam.appendChild(
-			doc.createTextNode(QString::number(paramValue(index))));
-		eParams.appendChild(eParam);
-	}
-	ePreset.appendChild(eParams);
-	doc.appendChild(ePreset);
-
-	QFile file(sFilename);
-	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		QTextStream(&file) << doc.toString();
-		file.close();
-	}
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
-
-	QDir::setCurrent(currentDir.absolutePath());
 }
 
 
@@ -875,6 +857,19 @@ void drumkv1widget::openSample (void)
 }
 
 
+// Sample updater slot.
+void drumkv1widget::updateSampleNotify (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1widget::updateSampleNotify()");
+#endif
+
+	drumkv1 *pDrumk = instance();
+	if (pDrumk)
+		updateSample(pDrumk->sample());
+}
+
+
 // Sample file reset.
 void drumkv1widget::clearSampleFile (void)
 {
@@ -901,7 +896,7 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 	if (pDrumk == NULL)
 		return;
 
-	int note = currentNote();
+	const int note = currentNote();
 	if (note < 0)
 		return;
 
@@ -948,7 +943,7 @@ int drumkv1widget::currentNote (void) const
 
 QString drumkv1widget::currentNoteName (void) const
 {
-	int note = currentNote();
+	const int note = currentNote();
 	return (note < 0 ? tr("(None)") : completeNoteName(note));
 }
 
@@ -1062,7 +1057,7 @@ bool drumkv1widget::queryClose (void)
 // Reload all elements.
 void drumkv1widget::refreshElements (void)
 {
-	bool bBlockSignals = m_ui.Elements->blockSignals(true);
+	const bool bBlockSignals = m_ui.Elements->blockSignals(true);
 
 	if (m_ui.Elements->instance() == NULL)
 		m_ui.Elements->setInstance(instance());
@@ -1095,7 +1090,7 @@ void drumkv1widget::clearElements (void)
 // Element activation.
 void drumkv1widget::activateElement ( bool bOpenSample )
 {
-	int iCurrentNote = currentNote();
+	const int iCurrentNote = currentNote();
 	if (iCurrentNote < 0)
 		return;
 
@@ -1204,10 +1199,8 @@ void drumkv1widget::bpmSyncChanged (void)
 	if (pDrumk) {
 		float *pBpmSync = pDrumk->paramPort(drumkv1::DEL1_BPMSYNC);
 		if (pBpmSync) {
-			const bool bBpmSync0
-				= (*pBpmSync > 0.0f);
-			const bool bBpmSync1
-				= (m_ui.Del1BpmKnob->minimum() >= m_ui.Del1BpmKnob->value());
+			const bool bBpmSync0 = (*pBpmSync > 0.0f);
+			const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
 			if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
 				*pBpmSync = (bBpmSync1 ? 1.0f : 0.0f);
 		}
@@ -1250,6 +1243,16 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 
 
 // Menu actions.
+void drumkv1widget::helpUseNativeDialogs ( bool bOn )
+{
+	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
+	if (pConfig) {
+		pConfig->bUseNativeDialogs = bOn;
+		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+	}
+}
+
+
 void drumkv1widget::helpAbout (void)
 {
 	// About...
@@ -1298,6 +1301,7 @@ void drumkv1widget::helpAbout (void)
 	QMessageBox::about(this, tr("About") + " " DRUMKV1_TITLE, sText);
 }
 
+
 void drumkv1widget::helpAboutQt (void)
 {
 	// About Qt...
