@@ -28,25 +28,9 @@
 
 #include "drumkv1.h"
 
-#include <QHash>
 
-// forward decl.
-class QDomElement;
-class QDomDocument;
-
-
-//-------------------------------------------------------------------------
-// drumkv1_map_path - abstract/absolute path functors.
-
-class drumkv1_map_path
-{
-public:
-
-	virtual QString absolutePath(const QString& sAbstractPath) const
-		{ return sAbstractPath; }
-	virtual QString abstractPath(const QString& sAbsolutePath) const
-		{ return sAbsolutePath; }
-};
+// forward decls.
+class drumkv1_sched_notifier;
 
 
 //-------------------------------------------------------------------------
@@ -62,6 +46,9 @@ public:
 	// Constructor
 	drumkv1widget(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
 
+	// Destructor.
+	~drumkv1widget();
+
 	// Param port accessors.
 	void setParamValue(
 		drumkv1::ParamIndex index, float fValue, bool bDefault = false);
@@ -107,7 +94,7 @@ protected slots:
 	// Sample openner.
 	void openSample();
 
-	// Sample loader slot.
+	// Sample loader.
 	void loadSample(const QString& sFilename);
 
 	// All element clear.
@@ -122,6 +109,9 @@ protected slots:
 	// Element deactivation.
 	void resetElement(void);
 
+	// Sample updater.
+	void updateSampleNotify();
+
 	// Common context menu.
 	void contextMenuRequest(const QPoint& pos);
 
@@ -135,6 +125,8 @@ protected slots:
 	void bpmSyncChanged();
 
 	// Menu actions.
+	void helpUseNativeDialogs(bool bOn);
+
 	void helpAbout();
 	void helpAboutQt();
 
@@ -149,6 +141,9 @@ protected:
 	// Reset swap params A/B group.
 	void resetSwapParams();
 
+	// Initialize param values.
+	void updateParamValues(uint32_t nparams);
+
 	// Reset all param/knob default values.
 	void resetParamValues(uint32_t nparams);
 	void resetParamKnobs(uint32_t nparams);
@@ -182,6 +177,8 @@ private:
 
 	drumkv1widget_config m_config;
 
+	drumkv1_sched_notifier *m_sched_notifier;
+
 	QHash<drumkv1::ParamIndex, drumkv1widget_knob *> m_paramKnobs;
 	QHash<drumkv1widget_knob *, drumkv1::ParamIndex> m_knobParams;
 
