@@ -27,7 +27,7 @@
 // Forward declarations.
 class QLabel;
 class QDial;
-class QSpinBox;
+class QDoubleSpinBox;
 class QComboBox;
 
 
@@ -47,15 +47,13 @@ public:
 	void setText(const QString& sText);
 	QString text() const;
 
-	float value() const;
-
 	virtual void setMaximum(float fMaximum);
 	float maximum() const;
 
 	virtual void setMinimum(float fMinimum);
 	float minimum() const;
 
-	void setSingleStep(float fSingleStep);
+	virtual void setSingleStep(float fSingleStep);
 	float singleStep() const;
 
 	void resetDefaultValue();
@@ -63,6 +61,7 @@ public:
 	float defaultValue() const;
 
 	virtual QString valueText() const;
+	virtual float value() const;
 
 	void setScale(float fScale);
 	float scale() const;
@@ -87,8 +86,8 @@ protected:
 	void mousePressEvent(QMouseEvent *pMouseEvent);
 
 	// Scale/value converters.
-	int scaleFromValue(float fValue) const;
-	float valueFromScale(int iScale) const;
+	float scaleFromValue(float fValue) const;
+	float valueFromScale(float fScale) const;
 
 private:
 
@@ -117,26 +116,38 @@ public:
 	// Constructor.
 	drumkv1widget_spin(QWidget *pParent = 0);
 
+	// Virtual accessors.
 	void setMaximum(float fMaximum);
 	void setMinimum(float fMinimum);
 
+	void setSingleStep(float fSingleStep);
+
+	QString valueText() const;
+	float value() const;
+
+	// Specialized accessors.
 	void setSpecialValueText(const QString& sText);
 	QString specialValueText() const;
 
+	bool isSpecialValue() const;
+
+	void setDecimals(int iDecimals);
+	int decimals() const;
+
 public slots:
 
-	// Virtual accessors.
+	// Virtual accessor.
 	void setValue(float fValue, bool bDefault = false);
 
 protected slots:
 
 	// Change slot.
-	void spinBoxValueChanged(int);
+	void spinBoxValueChanged(double);
 
 private:
 
 	// Widget members.
-	QSpinBox *m_pSpinBox;
+	QDoubleSpinBox *m_pSpinBox;
 };
 
 
@@ -152,12 +163,14 @@ public:
 	// Constructor.
 	drumkv1widget_combo(QWidget *pParent = 0);
 
-	// Accessors.
+	// Virtual accessors.
+	QString valueText() const;
+	float value() const;
+
+	// Specialized accessors.
 	void insertItems(int iIndex, const QStringList& items);
 	void clear();
 
-	QString valueText() const;
-
 public slots:
 
 	// Virtual accessor.
