@@ -25,12 +25,8 @@
 
 #include "lv2/lv2plug.in/ns/ext/instance-access/instance-access.h"
 
-#include <QSocketNotifier>
-
-#ifdef CONFIG_LV2_EXTERNAL_UI
 #include <QApplication>
 #include <QCloseEvent>
-#endif
 
 
 //-------------------------------------------------------------------------
@@ -45,34 +41,22 @@ drumkv1widget_lv2::drumkv1widget_lv2 ( drumkv1_lv2 *pDrumk,
 	m_controller = controller;
 	m_write_function = write_function;
 
-	// Update notifier setup.
-	m_pUpdateNotifier = new QSocketNotifier(
-		m_pDrumk->update_fds(1), QSocketNotifier::Read, this);
-
 #ifdef CONFIG_LV2_EXTERNAL_UI
 	m_external_host = NULL;
 #endif
-	
+#ifdef CONFIG_LV2_UI_IDLE
+	m_bIdleClosed = false;
+#endif
+
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
 		m_params_def[i] = true;
 
-	QObject::connect(m_pUpdateNotifier,
-		SIGNAL(activated(int)),
-		SLOT(updateNotify()));
-
 	// Initial update, always...
 	refreshElements();
 	activateElement();
 }
 
 
-// Destructor.
-drumkv1widget_lv2::~drumkv1widget_lv2 (void)
-{
-	delete m_pUpdateNotifier;
-}
-
-
 // Synth engine accessor.
 drumkv1 *drumkv1widget_lv2::instance (void) const
 {
@@ -95,18 +79,36 @@ const LV2_External_UI_Host *drumkv1widget_lv2::externalHost (void) const
 	return m_external_host;
 }
 
+#endif	// CONFIG_LV2_EXTERNAL_UI
+
+
+#ifdef CONFIG_LV2_UI_IDLE
+
+bool drumkv1widget_lv2::isIdleClosed (void) const
+{
+	return m_bIdleClosed;
+}
+
+#endif	// CONFIG_LV2_UI_IDLE
+
+
+// Close event handler.
 void drumkv1widget_lv2::closeEvent ( QCloseEvent *pCloseEvent )
 {
 	drumkv1widget::closeEvent(pCloseEvent);
 
+#ifdef CONFIG_LV2_UI_IDLE
+	if (pCloseEvent->isAccepted())
+		m_bIdleClosed = true;
+#endif
+#ifdef CONFIG_LV2_EXTERNAL_UI
 	if (m_external_host && m_external_host->ui_closed) {
 		if (pCloseEvent->isAccepted())
 			m_external_host->ui_closed(m_controller);
 	}
+#endif
 }
 
-#endif	// CONFIG_LV2_EXTERNAL_UI
-
 
 // Plugin port event notification.
 void drumkv1widget_lv2::port_event ( uint32_t port_index,
@@ -135,30 +137,15 @@ void drumkv1widget_lv2::updateParam (
 }
 
 
-// Update notification slot.
-void drumkv1widget_lv2::updateNotify (void)
-{
-#ifdef CONFIG_DEBUG
-	qDebug("drumkv1widget_lv2::updateNotify()");
-#endif
-
-	updateSample(m_pDrumk->sample());
-
-	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		const float *pfValue = m_pDrumk->paramPort(index);
-		setParamValue(index, (pfValue ? *pfValue : 0.0f));
-	}
-
-	m_pDrumk->update_reset();
-}
-
-
 //-------------------------------------------------------------------------
 // drumkv1widget_lv2 - LV2 UI desc.
 //
 
 
+static QApplication *drumkv1_lv2ui_qapp_instance = NULL;
+static unsigned int  drumkv1_lv2ui_qapp_refcount = 0;
+
+
 static LV2UI_Handle drumkv1_lv2ui_instantiate (
 	const LV2UI_Descriptor *, const char *, const char *,
 	LV2UI_Write_Function write_function,
@@ -177,6 +164,13 @@ static LV2UI_Handle drumkv1_lv2ui_instantiate (
 	if (pSampl == NULL)
 		return NULL;
 
+	if (qApp == NULL && drumkv1_lv2ui_qapp_instance == NULL) {
+		static int s_argc = 1;
+		static const char *s_argv[] = { __func__, NULL };
+		drumkv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
+	}
+	drumkv1_lv2ui_qapp_refcount++;
+
 	drumkv1widget_lv2 *pWidget
 		= new drumkv1widget_lv2(pSampl, controller, write_function);
 	*widget = pWidget;
@@ -187,8 +181,13 @@ static LV2UI_Handle drumkv1_lv2ui_instantiate (
 static void drumkv1_lv2ui_cleanup ( LV2UI_Handle ui )
 {
 	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
-	if (pWidget)
+	if (pWidget) {
 		delete pWidget;
+		if (--drumkv1_lv2ui_qapp_refcount == 0 && drumkv1_lv2ui_qapp_instance) {
+			delete drumkv1_lv2ui_qapp_instance;
+			drumkv1_lv2ui_qapp_instance = NULL;
+		}
+	}
 }
 
 static void drumkv1_lv2ui_port_event (
@@ -200,8 +199,74 @@ static void drumkv1_lv2ui_port_event (
 		pWidget->port_event(port_index, buffer_size, format, buffer);
 }
 
-static const void *drumkv1_lv2ui_extension_data ( const char * )
+
+#ifdef CONFIG_LV2_UI_IDLE
+
+int drumkv1_lv2ui_idle ( LV2UI_Handle ui )
 {
+	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
+	if  (pWidget && !pWidget->isIdleClosed()) {
+		QApplication::processEvents();
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+static const LV2UI_Idle_Interface drumkv1_lv2ui_idle_interface =
+{
+	drumkv1_lv2ui_idle
+};
+
+#endif	// CONFIG_LV2_UI_IDLE
+
+
+#ifdef CONFIG_LV2_UI_SHOW
+
+int drumkv1_lv2ui_show ( LV2UI_Handle ui )
+{
+	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
+	if (pWidget) {
+		pWidget->show();
+		pWidget->raise();
+		pWidget->activateWindow();
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+int drumkv1_lv2ui_hide ( LV2UI_Handle ui )
+{
+	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
+	if (pWidget) {
+		pWidget->hide();
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+static const LV2UI_Show_Interface drumkv1_lv2ui_show_interface =
+{
+	drumkv1_lv2ui_show,
+	drumkv1_lv2ui_hide
+};
+
+#endif	// CONFIG_LV2_UI_IDLE
+
+static const void *drumkv1_lv2ui_extension_data ( const char *uri )
+{
+#ifdef CONFIG_LV2_UI_IDLE
+	if (::strcmp(uri, LV2_UI__idleInterface) == 0)
+		return (void *) &drumkv1_lv2ui_idle_interface;
+	else
+#endif
+#ifdef CONFIG_LV2_UI_SHOW
+	if (::strcmp(uri, LV2_UI__showInterface) == 0)
+		return (void *) &drumkv1_lv2ui_show_interface;
+	else
+#endif
 	return NULL;
 }
 
@@ -211,21 +276,15 @@ static const void *drumkv1_lv2ui_extension_data ( const char * )
 struct drumkv1_lv2ui_external_widget
 {
 	LV2_External_UI_Widget external;
-	static QApplication   *app_instance;
-	static unsigned int    app_refcount;
 	drumkv1widget_lv2     *widget;
 };
 
-QApplication *drumkv1_lv2ui_external_widget::app_instance = NULL;
-unsigned int  drumkv1_lv2ui_external_widget::app_refcount = 0;
-
-
 static void drumkv1_lv2ui_external_run ( LV2_External_UI_Widget *ui_external )
 {
 	drumkv1_lv2ui_external_widget *pExtWidget
 		= (drumkv1_lv2ui_external_widget *) (ui_external);
-	if (pExtWidget && pExtWidget->app_instance)
-		pExtWidget->app_instance->processEvents();
+	if (pExtWidget)
+		QApplication::processEvents();
 }
 
 static void drumkv1_lv2ui_external_show ( LV2_External_UI_Widget *ui_external )
@@ -271,14 +330,14 @@ static LV2UI_Handle drumkv1_lv2ui_external_instantiate (
 	if (pDrumk == NULL)
 		return NULL;
 
-	drumkv1_lv2ui_external_widget *pExtWidget = new drumkv1_lv2ui_external_widget;
-	if (qApp == NULL && pExtWidget->app_instance == NULL) {
+	if (qApp == NULL && drumkv1_lv2ui_qapp_instance == NULL) {
 		static int s_argc = 1;
 		static const char *s_argv[] = { __func__, NULL };
-		pExtWidget->app_instance = new QApplication(s_argc, (char **) s_argv, true);
+		drumkv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
 	}
-	pExtWidget->app_refcount++;
+	drumkv1_lv2ui_qapp_refcount++;
 
+	drumkv1_lv2ui_external_widget *pExtWidget = new drumkv1_lv2ui_external_widget;
 	pExtWidget->external.run  = drumkv1_lv2ui_external_run;
 	pExtWidget->external.show = drumkv1_lv2ui_external_show;
 	pExtWidget->external.hide = drumkv1_lv2ui_external_hide;
@@ -296,11 +355,11 @@ static void drumkv1_lv2ui_external_cleanup ( LV2UI_Handle ui )
 	if (pExtWidget) {
 		if (pExtWidget->widget)
 			delete pExtWidget->widget;
-		if (--pExtWidget->app_refcount == 0 && pExtWidget->app_instance) {
-			delete pExtWidget->app_instance;
-			pExtWidget->app_instance = NULL;
-		}
 		delete pExtWidget;
+		if (--drumkv1_lv2ui_qapp_refcount == 0 && drumkv1_lv2ui_qapp_instance) {
+			delete drumkv1_lv2ui_qapp_instance;
+			drumkv1_lv2ui_qapp_instance = NULL;
+		}
 	}
 }
 
