@@ -1,7 +1,7 @@
 // drumkv1widget_preset.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -125,7 +125,7 @@ void drumkv1widget_preset::clearPreset (void)
 	if (pConfig)
 		pConfig->sPreset.clear();
 
-	bool bBlockSignals = m_pComboBox->blockSignals(true);
+	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->clearEditText();
 	m_pComboBox->blockSignals(bBlockSignals);
 }
@@ -133,7 +133,7 @@ void drumkv1widget_preset::clearPreset (void)
 
 void drumkv1widget_preset::setPreset ( const QString& sPreset )
 {
-	bool bBlockSignals = m_pComboBox->blockSignals(true);
+	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->setEditText(sPreset);
 	m_pComboBox->blockSignals(bBlockSignals);
 }
@@ -241,8 +241,11 @@ void drumkv1widget_preset::openPreset (void)
 	const QString& sTitle  = tr("Open Preset") + " - " DRUMKV1_TITLE;
 	const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
 #if 1//QT_VERSION < 0x040400
+	QFileDialog::Options options = 0;
+	if (pConfig->bDontUseNativeDialogs)
+		options |= QFileDialog::DontUseNativeDialog;
 	sFilename = QFileDialog::getOpenFileName(parentWidget(),
-		sTitle, pConfig->sPresetDir, sFilter);
+		sTitle, pConfig->sPresetDir, sFilter, NULL, options);
 #else
 	QFileDialog fileDialog(nativeParentWidget(),
 		sTitle, pConfig->sPresetDir, sFilter);
@@ -252,6 +255,8 @@ void drumkv1widget_preset::openPreset (void)
 	QList<QUrl> urls(fileDialog.sidebarUrls());
 	urls.append(QUrl::fromLocalFile(pConfig->sPresetDir));
 	fileDialog.setSidebarUrls(urls);
+	if (pConfig->bDontUseNativeDialogs)
+		fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
 	if (fileDialog.exec())
 		sFilename = fileDialog.selectedFiles().first();
 #endif
@@ -296,8 +301,11 @@ void drumkv1widget_preset::savePreset ( const QString& sPreset )
 		const QString& sTitle  = tr("Save Preset") + " - " DRUMKV1_TITLE;
 		const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
 	#if 1//QT_VERSION < 0x040400
+		QFileDialog::Options options = 0;
+		if (pConfig->bDontUseNativeDialogs)
+			options |= QFileDialog::DontUseNativeDialog;
 		sFilename = QFileDialog::getSaveFileName(parentWidget(),
-			sTitle, sFilename, sFilter);
+			sTitle, sFilename, sFilter, NULL, options);
 	#else
 		QFileDialog fileDialog(nativeParentWidget(),
 			sTitle, sFilename, sFilter);
@@ -307,6 +315,8 @@ void drumkv1widget_preset::savePreset ( const QString& sPreset )
 		QList<QUrl> urls(fileDialog.sidebarUrls());
 		urls.append(QUrl::fromLocalFile(pConfig->sPresetDir));
 		fileDialog.setSidebarUrls(urls);
+		if (pConfig->bDontUseNativeDialogs)
+			fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
 		if (fileDialog.exec())
 			sFilename = fileDialog.selectedFiles().first();
 	#endif
@@ -376,8 +386,7 @@ void drumkv1widget_preset::deletePreset (void)
 void drumkv1widget_preset::resetPreset (void)
 {
 	const QString& sPreset = m_pComboBox->currentText();
-
-	bool bLoadPreset = (!sPreset.isEmpty()
+	const bool bLoadPreset = (!sPreset.isEmpty()
 		&& m_pComboBox->findText(sPreset) >= 0);
 	if (bLoadPreset && !queryPreset())
 		return;
@@ -395,7 +404,7 @@ void drumkv1widget_preset::resetPreset (void)
 // Widget refreshner-loader.
 void drumkv1widget_preset::refreshPreset (void)
 {
-	bool bBlockSignals = m_pComboBox->blockSignals(true);
+	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 
 	const QString sOldPreset = m_pComboBox->currentText();
 	const QIcon icon(":/images/drumkv1_preset.png");
@@ -414,7 +423,7 @@ void drumkv1widget_preset::refreshPreset (void)
 		pConfig->endGroup();
 	}
 
-	int iIndex = m_pComboBox->findText(sOldPreset);
+	const int iIndex = m_pComboBox->findText(sOldPreset);
 	if (iIndex >= 0)
 		m_pComboBox->setCurrentIndex(iIndex);
 	else
